/* debug.c   debugging output to file and screen */

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>

#include "debug.h"

static int current_screen_debug_level;		/* one of DEBUG_LEVEL constants */
static int current_file_debug_level;		/* one of DEBUG_LEVEL constants */

/* should be called at the beginning of the program */
void initdebug(int screen_debug_level, int file_debug_level)
{
	FILE *df;
	time_t t;
	current_screen_debug_level = screen_debug_level;
	current_file_debug_level = file_debug_level;
	
	if (current_file_debug_level > DL_NONE)
	{
		df = fopen(DEBUG_FILENAME, "w+");
		if (df == NULL)
		{
			perror("Cannot open debug log file");
			exit(1);
		}
		time(&t);
		fprintf(df, "Cargo debug log file, debug level %d, started at %s\n", current_file_debug_level, ctime(&t));
		fclose(df);
	}
}

/* prints debug string message to the screen and file, 
 * if it matches the current debug level */
void debugmsg(int debuglevel, char *msg)
{
	FILE *df;
	
	if (debuglevel & current_screen_debug_level)
		printf(msg);
	if (debuglevel & current_file_debug_level)
	{
		df = fopen(DEBUG_FILENAME, "a");
		if (df == NULL)
			perror("WARNING: Cannot open debug log file");
		else
		{
			fprintf(df, msg);
			fclose(df);
		}
	}
}

/* prints debug string message and one integer to the screen and file, 
 * if it matches the current debug level */
void debugmsg1int(int debuglevel, char *msg, int num)
{
	FILE *df;

	if (debuglevel & current_screen_debug_level)
		printf(msg, num);
	if (debuglevel & current_file_debug_level)
	{
		df = fopen(DEBUG_FILENAME, "a");
		if (df == NULL)
			perror("WARNING: Cannot open debug log file");
		else
		{
			fprintf(df, msg, num);
			fclose(df);
		}
	}
}

/* prints debug string message and 2 integers to the screen and file, 
 * if it matches the current debug level */
void debugmsg2int(int debuglevel, char *msg, int num1, int num2)
{
	FILE *df;

	if (debuglevel & current_screen_debug_level)
		printf(msg, num1, num2);
	if (debuglevel & current_file_debug_level)
	{
		df = fopen(DEBUG_FILENAME, "a");
		if (df == NULL)
			perror("WARNING: Cannot open debug log file");
		else
		{
			fprintf(df, msg, num1, num2);
			fclose(df);
		}
	}
}

/* prints debug string message and 1 double to the screen and file, 
 * if it matches the current debug level */
void debugmsg1double(int debuglevel, char *msg, double num)
{
	FILE *df;

	if (debuglevel & current_screen_debug_level)
		printf(msg, num);
	if (debuglevel & current_file_debug_level)
	{
		df = fopen(DEBUG_FILENAME, "a");
		if (df == NULL)
			perror("WARNING: Cannot open debug log file");
		else
		{
			fprintf(df, msg, num);
			fclose(df);
		}
	}
}

/* prints debug string message and 2 doubles to the screen and file, 
 * if it matches the current debug level */
void debugmsg2double(int debuglevel, char *msg, double num1, double num2)
{
	FILE *df;

	if (debuglevel & current_screen_debug_level)
		printf(msg, num1, num2);
	if (debuglevel & current_file_debug_level)
	{
		df = fopen(DEBUG_FILENAME, "a");
		if (df == NULL)
			perror("WARNING: Cannot open debug log file");
		else
		{
			fprintf(df, msg, num1, num2);
			fclose(df);
		}
	}
}

/* prints debug string message and 1 long int to the screen and file, 
 * if it matches the current debug level */
void debugmsg1long(int debuglevel, char *msg, long num)
{
	FILE *df;

	if (debuglevel & current_screen_debug_level)
		printf(msg, num);
	if (debuglevel & current_file_debug_level)
	{
		df = fopen(DEBUG_FILENAME, "a");
		if (df == NULL)
			perror("WARNING: Cannot open debug log file");
		else
		{
			fprintf(df, msg, num);
			fclose(df);
		}
	}
}

/* prints debug string message and 2 long ints to the screen and file, 
 * if it matches the current debug level */
void debugmsg2long(int debuglevel, char *msg, long num1, long num2)
{
	FILE *df;

	if (debuglevel & current_screen_debug_level)
		printf(msg, num1, num2);
	if (debuglevel & current_file_debug_level)
	{
		df = fopen(DEBUG_FILENAME, "a");
		if (df == NULL)
			perror("WARNING: Cannot open debug log file");
		else
		{
			fprintf(df, msg, num1, num2);
			fclose(df);
		}
	}
}
