/* simulator_internals.h   Internal type definitions constants and fn. prototypes related to the simulator */

#ifndef    __simulator_internal_h__
#define __simulator_internal_h__


/* what is the robot doing right now */
#define MOVING_STOPPED			1		/* standing still */
#define MOVING_FWD_STRAIGHT		2		/* moving straight forward */
#define MOVING_BWD_STRAIGHT		3		/* moving straight backwards */
#define MOVING_FWD_LEFT			4		/* turning along a circle to the left, in forward direction */
#define MOVING_BWD_LEFT			5		/* same circle, but backwards direction (i.e. actually turning right from the point of view of motion */
#define MOVING_FWD_RIGHT		6		/* ... */
#define MOVING_BWD_RIGHT		7
#define MOVING_ROTATING_LEFT	8		/* rotating at a spot to the left */
#define MOVING_ROTATING_RIGHT	9

#define pi				(double)3.14159265359
#define RADIAN90DEG		(pi / (double)2.0)
#define RADIAN180DEG	pi
#define RADIAN270DEG	((double)3.0 * pi / (double)2.0)

#define BEHIND_THE_HILL	((double)4.0 * pi)	/* used as a special return value when angle 0..2pi is normally returned */

#define epsilon			0.0001

/* time event type */
typedef struct {
  long time;         /* simulation time of the event */
  int script_index;  /* pointer to the script array, that script will be executed, if the next event
                        is obstacle or wall, script_index is set to -1 */
  int active_index;	 /* in case of active area we have to remember also which one it is to be able
                        to check its other parameters in case of activation */
} time_event_str;

/* variables used by the viewer, defined in simulator.c */
extern int robot_moving;
extern double robot_heading;	
extern double robot_x;
extern double robot_y;
extern double robot_radius;
extern double robot_speed;		
extern long new_simulation_time;
extern int obstcnt;
extern double obs_x[MAX_OBSTCNT];
extern double obs_y[MAX_OBSTCNT];
extern double obs_w[MAX_OBSTCNT];
extern double obs_h[MAX_OBSTCNT];
extern int markcnt;
extern double mark_x[MAX_MARKCNT];
extern double mark_y[MAX_MARKCNT];
extern double mark_w[MAX_MARKCNT];
extern double mark_h[MAX_MARKCNT];
extern int mark_value[MAX_MARKCNT];
extern int lightcnt;
extern double light_x[MAX_LIGHTCNT];
extern double light_y[MAX_LIGHTCNT];
extern double light_intensity[MAX_LIGHTCNT];
extern int light_state[MAX_LIGHTCNT];		
extern int activecnt;
extern double active_x[MAX_ACTIVECNT];
extern double active_y[MAX_ACTIVECNT];
extern double active_w[MAX_ACTIVECNT];
extern double active_h[MAX_ACTIVECNT];
extern char *moving_desc[10];
extern int	traj_moving[MAX_TRAJECTORY_LEN];
extern double traj_x[MAX_TRAJECTORY_LEN];
extern double traj_y[MAX_TRAJECTORY_LEN];
extern double traj_c_r[MAX_TRAJECTORY_LEN];
extern double traj_c_x[MAX_TRAJECTORY_LEN];
extern double traj_c_y[MAX_TRAJECTORY_LEN];
extern int traji;

/* function prototypes */

void update_robot_speed();
void update_simulation();
void preprocess_simulation();


#endif //__simulator_internal_h__